﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("PackingSequence OX", "An operator which crosses two PackingSequence representations using the OX-strategy.")]
  [StorableClass]
  public class PackingSequenceOrderCrossover : PackingSequenceCrossover, IPackingSequenceOperator {

    [StorableConstructor]
    protected PackingSequenceOrderCrossover(bool deserializing) : base(deserializing) { }
    protected PackingSequenceOrderCrossover(PackingSequenceOrderCrossover original, Cloner cloner) : base(original, cloner) { }
    public PackingSequenceOrderCrossover()
      : base() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PackingSequenceOrderCrossover(this, cloner);
    }

    public override PackingSequenceEncoding Cross(IRandom random, PackingSequenceEncoding parent1, PackingSequenceEncoding parent2)  {
      PackingSequenceEncoding result = new PackingSequenceEncoding();
      result.PackingSequence = OrderCrossover.Apply(random, parent1.PackingSequence, parent2.PackingSequence);
      return result;
    }
  }
}
